# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient.tests import utils
from seamicroclient.tests.v2 import fakes
from seamicroclient.v2 import system


cs = fakes.FakeClient()


class Systemstest(utils.TestCase):

    def test_list_system(self):
        pl = cs.system.list()
        cs.assert_called('GET', '/chassis/systems')
        [self.assertTrue(isinstance(s, system.System)) for s in pl]

    def test_switchover_system(self):
        cs.system.switchover(1)
        cs.assert_called('PUT', '/chassis/system/switchover')

    def test_writemem_system(self):
        cs.system.writemem(1)
        cs.assert_called('PUT', '/chassis/system/writeMem')

    def test_reload_system(self):
        cs.system.reload(1)
        cs.assert_called('PUT', '/chassis/system/reload')



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient import client
from seamicroclient.v2 import chassis
from seamicroclient.v2 import disks
from seamicroclient.v2 import fantrays
from seamicroclient.v2 import interfaces
from seamicroclient.v2 import pools
from seamicroclient.v2 import powersupplies
from seamicroclient.v2 import scards
from seamicroclient.v2 import servers
from seamicroclient.v2 import smcards
from seamicroclient.v2 import system
from seamicroclient.v2 import volumes


class Client(object):

    """
    Top-level object to access the Seamicro Chassis API.

    Create an instance with your creds::

        >>> client = Client(USERNAME, PASSWORD, AUTH_URL)

    Then call methods on its managers::

        >>> client.servers.list()
        ...
        >>> client.chassis.list()
        ...

    """

    def __init__(self, username, password, auth_url=None,
                 timeout=None, http_log_debug=False):
        self.servers = servers.ServerManager(self)
        self.pools = pools.PoolManager(self)
        self.volumes = volumes.VolumeManager(self)
        self.disks = disks.DiskManager(self)
        self.chassis = chassis.ChassisManager(self)
        self.fantrays = fantrays.FanTrayManager(self)
        self.interfaces = interfaces.InterfaceManager(self)
        self.powersupplies = powersupplies.PowerSupplyManager(self)
        self.scards = scards.ScardManager(self)
        self.smcards = smcards.SMCardManager(self)
        self.system = system.SystemManager(self)

        self.client = client.HTTPClient(username,
                                        password,
                                        auth_url=auth_url,
                                        timeout=timeout,
                                        http_log_debug=http_log_debug)

    def get_timings(self):
        return self.client.get_timings()

    def reset_timings(self):
        self.client.reset_timings()

    def authenticate(self):
        """
        Authenticate against the server.

        Normally this is called automatically when you first access the API,
        but you can call this method to force authentication right now.

        Returns on success; raises :exc:`exceptions.Unauthorized` if the
        credentials are wrong.
        """
        self.client.authenticate()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from seamicroclient import client as base_client
from seamicroclient.openstack.common.py3kcompat import urlutils
from seamicroclient.tests import fakes
from seamicroclient.tests import utils
from seamicroclient.v2 import client


class FakeClient(fakes.FakeClient, client.Client):

    def __init__(self, *args, **kwargs):
        client.Client.__init__(self, 'username', 'password',
                               'auth_url')
        self.client = FakeHTTPClient(**kwargs)


class FakeHTTPClient(base_client.HTTPClient):

    def __init__(self, **kwargs):
        self.username = 'username'
        self.password = 'password'
        self.auth_url = 'auth_url'
        self.callstack = []
        self.timings = 'timings'
        self.http_log_debug = 'http_log_debug'

    def _cs_request(self, url, method, **kwargs):
        # Check that certain things are called correctly
        if method in ['GET', 'DELETE']:
            assert 'body' not in kwargs
        elif method == 'PUT':
            assert 'body' in kwargs

        # Call the method
        args = urlutils.parse_qsl(urlutils.urlparse(url)[4])
        kwargs.update(args)
        munged_url = url.rsplit('?', 1)[0]
        munged_url = munged_url.strip('/').replace('/', '_').replace('.', '_')
        munged_url = munged_url.replace('-', '_')
        munged_url = munged_url.replace(' ', '_')

        callback = "%s_%s" % (method.lower(), munged_url)

        if not hasattr(self, callback):
            raise AssertionError('Called unknown API method: %s %s, '
                                 'expected fakes method name: %s' %
                                 (method, url, callback))

        # Note the call
        self.callstack.append((method, url, kwargs.get('body', None)))

        status, headers, body = getattr(self, callback)(**kwargs)
        r = utils.TestResponse({
            "status_code": status,
            "text": body,
            "headers": headers,
        })
        return r, body

    def get_servers_1(self, **kwargs):
        return (200, {}, {'id': 1234, 'name': 'sample-server'}
                )

    def get_servers(self, **kwargs):
        return (200, {}, {'0/0': {}, '1/0': {}})

    def put_servers_1(self, **kwargs):
        return (200, {}, {})

    def put_servers_1_nic_0_untaggedVlans(self, **kwargs):
        return (200, {}, {})

    def put_servers_1_nic_0_taggedVlans(self, **kwargs):
        return (200, {}, {})

    def get_storage_pools(self):
        return (200, {}, {'0/p0-0': {}, '0/p1-1': {}})

    def get_storage_pools_1(self):
        return (200, {}, {'0/p0-0': {}})

    def get_storage_volumes(self):
        return (200, {}, {'0/p0-0/1': {}, '0/p1-1/1': {}})

    def get_storage_volumes_1(self):
        return (200, {}, {'1': {}})

    def put_storage_volumes_0_p0_0_1(self, **kwargs):
        return (200, {}, {'0/p0-0/1': {}})

    def get_storage_volumes_0_p0_0_1(self, **kwargs):
        return (200, {}, {'0/p0-0/1': {}})

    def put_servers_1_vdisk_0(self, **kwargs):
        return (200, {}, {})

    def put_servers_1_vdisk_3(self, **kwargs):
        return (200, {}, {})

    def delete_servers_1_vdisk_0(self, **kwargs):
        return (200, {}, {})

    def get_storage_disks(self, **kwargs):
        return (200, {}, {})

    def get_storage_disks_1(self, **kwargs):
        return (200, {}, {'1': {}})

    def put_storage_disks_1(self, **kwargs):
        return (200, {}, {'1': {}})

    def get_chassis(self, **kwargs):
        return (200, {}, {})

    def put_chassis_system_writeMem(self, **kwargs):
        return (200, {}, {})

    def get_chassis_fanTray(self, **kwargs):
        return (200, {}, {})

    def get_chassis_fanTray_1(self, **kwargs):
        return (200, {}, {})

    def get_interfaces(self, **kwargs):
        return (200, {}, {})

    def get_interfaces_1(self, **kwargs):
        return (200, {}, {})

    def put_interfaces_1_shutdown(self, **kwargs):
        return (200, {}, {})

    def put_interfaces_1_vlans_taggedVlans(self, **kwargs):
        return (200, {}, {})

    def put_interfaces_1_vlans_untaggedVlans(self, **kwargs):
        return (200, {}, {})

    def put_storage_pools_1_pool_name(self, **kwargs):
        return (200, {}, {})

    def delete_storage_pools_1_pool_name(self, **kwargs):
        return (200, {}, {})

    def put_storage_pools_1(self, **kwargs):
        return (200, {}, {})

    def get_chassis_powersupply(self, **kwargs):
        return (200, {}, {})

    def get_chassis_powersupply_1(self, **kwargs):
        return (200, {}, {})

    def get_chassis_scard(self, **kwargs):
        return (200, {}, {})

    def get_chassis_scard_1(self, **kwargs):
        return (200, {}, {})

    def put_chassis_scard_1_mgmtMode(self, **kwargs):
        return (200, {}, {})

    def get_chassis_smcard(self, **kwargs):
        return (200, {}, {})

    def get_chassis_smcard_1(self, **kwargs):
        return (200, {}, {})

    def get_chassis_systems(self, **kwargs):
        return (200, {}, {})

    def put_chassis_system_switchover(self, **kwargs):
        return (200, {}, {})

    def put_chassis_system_reload(self, **kwargs):
        return (200, {}, {})



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import time

from seamicroclient.tests import utils
from seamicroclient.v2 import Client


STATUS_WAIT_TIMEOUT = 30
BUILD_INTERVAL = 10

SERVER_ID = '0/0'
UNTAGGED_VLAN_ID = '7'
TAGGED_VLAN_ID = '17'

cs = Client("admin", "seamicro", "http://chassis/v2.0")


class FunctionalException(Exception):
    pass


class ServersTest(utils.TestCase):

    @staticmethod
    def wait_for_server_status(s, active):
        start_time = int(time.time())
        while True:
            timed_out = int(time.time()) - start_time > STATUS_WAIT_TIMEOUT
            if timed_out:
                raise FunctionalException()
            time.sleep(BUILD_INTERVAL)
            s = s.refresh()
            if s.active == active:
                return

    @staticmethod
    def create_volume(size=1):
        pool = cs.pools.list()[0]
        return cs.volumes.create(size, pool)

    def test_list_servers(self):
        sl = cs.servers.list()
        self.assertTrue(len(sl) > 0)

    def test_get_server(self):
        s = cs.servers.get(SERVER_ID)
        self.assertEqual(s.id, SERVER_ID)

    def test_power_on_power_off(self):
        s = cs.servers.get(SERVER_ID)
        self.assertEqual(s.id, SERVER_ID)
        if s.active:
            s.power_off()
            self.wait_for_server_status(s, active=False)
            s = s.refresh()
            self.assertEqual(s.active, False)
            s.power_on()
        else:
            s.power_on()
            self.wait_for_server_status(s, active=True)
            s = s.refresh()
            self.assertEqual(s.active, True)
            s.power_off()

    def test_reset(self):
        s = cs.servers.get(SERVER_ID)
        self.assertEqual(s.id, SERVER_ID)
        s.reset()
        self.wait_for_server_status(s, active=True)
        s = s.refresh()
        self.assertEqual(s.active, True)

    def test_attach_detach_volume(self):
        volume_id = self.create_volume()
        server = cs.servers.list()[0]
        server.detach_volume()
        server.attach_volume(volume_id)
        server = server.refresh()
        self.assertEqual(server.vdisk['0'], volume_id)
        server.detach_volume()
        cs.volumes.delete(volume_id)

    def test_set_tagged_vlan(self):
        server = cs.servers.list()[0]
        server.unset_tagged_vlan(TAGGED_VLAN_ID)
        server = server.refresh(10)
        server.set_tagged_vlan(TAGGED_VLAN_ID)
        server = server.refresh(10)
        self.assertTrue(TAGGED_VLAN_ID in server.nic['0']['taggedVlan'])
        server.unset_tagged_vlan(TAGGED_VLAN_ID)

    def test_unset_tagged_vlan(self):
        server = cs.servers.list()[0]
        server.unset_tagged_vlan(TAGGED_VLAN_ID)
        server = server.refresh(10)
        server.set_tagged_vlan(TAGGED_VLAN_ID)
        server = server.refresh(10)
        server.unset_tagged_vlan(TAGGED_VLAN_ID)
        server = server.refresh(10)
        self.assertTrue(TAGGED_VLAN_ID not in server.nic['0']['taggedVlan'])

    def test_unset_untagged_vlan(self):
        server = cs.servers.list()[0]
        server.unset_untagged_vlan(UNTAGGED_VLAN_ID)
        server = server.refresh(10)
        server.set_untagged_vlan(UNTAGGED_VLAN_ID)
        server = server.refresh(10)
        server.unset_untagged_vlan(UNTAGGED_VLAN_ID)
        server = server.refresh(10)
        self.assertTrue(UNTAGGED_VLAN_ID not in
                        server.nic['0']['untaggedVlan'])

    def test_set_untagged_vlan(self):
        server = cs.servers.list()[0]
        server.unset_untagged_vlan(UNTAGGED_VLAN_ID)
        server = server.refresh(10)
        server.set_untagged_vlan(UNTAGGED_VLAN_ID)
        server = server.refresh(10)
        self.assertTrue(UNTAGGED_VLAN_ID in server.nic['0']['untaggedVlan'])
        server.unset_untagged_vlan(UNTAGGED_VLAN_ID)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Pool interface.
"""


from seamicroclient import base


class Pool(base.Resource):
    HUMAN_ID = True

    def delete(self, **kwargs):
        return self.manager.delete(self, **kwargs)

    def mount(self, **kwargs):
        return self.manager.mount(self, **kwargs)


class PoolManager(base.ManagerWithFind):
    resource_class = Pool

    def get(self, pool):
        """
        Get a pool.

        :param pool: ID of the :class:`Pool` to get.
        :rtype: :class:`Pool`
        """
        return self._get(base.getid(pool),
                         "/storage/pools/%s" % base.getid(pool))

    def list(self, filters=None):
        """
        Get a list of pools.

        :rtype: list of :class:`Pool`
        """
        return self._list("/storage/pools", filters=filters)

    def create(self, slot, pool_name, disks, raid_level=0, **kwargs):
        """
        Create a pool on the given scard slot using disks of that slot.

        :rtype: Instance of :class:`Pool`
        """
        disks = map(lambda x: str(x), disks)
        body = {'disks': ','.join(disks), 'raidLevel': raid_level}
        url = '/storage/pools/%s/%s' % (base.getid(slot), pool_name)
        return self.api.client.put(url, body=body)

    def delete(self, pool, **kwargs):
        """
        Delete the specified pool.
        """
        url = '/storage/pools/%s' % base.getid(pool)
        return self.api.client.delete(url)

    def mount(self, pool, **kwargs):
        """
        Mount the specified Pool
        """
        return self._action("mount", pool, **kwargs)

    def unmount(self, pool, **kwargs):
        """
        UnMount the specified Pool
        """
        return self._action("unmount", pool, **kwargs)

    def _action(self, action, pool, info=None, **kwargs):
        """
        Perform a pool "action" -- .
        """
        body = {"action": action}
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/storage/pools/%s' % base.getid(pool)
        return self.api.client.put(url, body=body)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base utilities to build API operation managers and objects on top of.
"""

import abc
import time

import six

from seamicroclient import exceptions
from seamicroclient.openstack.common import strutils
from seamicroclient import utils


def getid(obj):
    """
    Abstracts the common pattern of allowing both an object or an object's ID
    as a parameter when dealing with relationships.
    """
    try:
        return obj.id
    except AttributeError:
        return obj


class Manager(utils.HookableMixin):

    """
    Managers interact with a particular type of API (servers, storage
    etc.) and provide CRUD operations for them.
    """
    resource_class = None

    def __init__(self, api):
        self.api = api

    def _list(self, url, body=None, filters=None):
        if body:
            _resp, body = self.api.client.post(url, body=body)
        else:
            _resp, body = self.api.client.get(url)

        obj_class = self.resource_class

        data = body
        output = []
        for k, v in data.iteritems():
            if data[k]:
                if type(v) != dict:
                    output.append(obj_class(self, data, loaded=True))
                    break
                v.update({'id': k})
                output.append(obj_class(self, v, loaded=True))

        filtered_output = set()
        if filters is not None:
            for k, v in filters.iteritems():
                for item in output:
                    if isinstance(v, basestring):
                        if v in getattr(item, k):
                            output.add(item)
                        else:
                            if item in output:
                                output.remove(item)
                    elif isinstance(v, int):
                        if v == getattr(item, k):
                            output.add(item)
                        else:
                            if item in output:
                                output.remove(item)
                    else:
                        continue
            return filtered_output
        return output

    def _get(self, id, url):
        _resp, body = self.api.client.get(url)
        body.update({'id': id})
        return self.resource_class(self, body)

    def _create(self, url, body, return_raw=False, **kwargs):
        self.run_hooks('modify_body_for_create', body, **kwargs)
        _resp, body = self.api.client.post(url, body=body)
        if isinstance(body, basestring):
            return body.partition('/')[-1]
        if return_raw:
            return body
        for k, v in body.iteritems():
            v.update({'id': k})
            return self.resource_class(self, v)

    def _delete(self, url):
        _resp, _body = self.api.client.delete(url)

    def _update(self, url, body, **kwargs):
        self.run_hooks('modify_body_for_update', body, **kwargs)
        _resp, body = self.api.client.put(url, body=body)
        if body:
            if isinstance(body, basestring):
                return body.partition('/')[-1]

            if body == kwargs.get('action'):
                return
            for k, v in body.iteritems():
                v.update({'id': k})
                return self.resource_class(self, v)


@six.add_metaclass(abc.ABCMeta)
class ManagerWithFind(Manager):

    """
    Like a `Manager`, but with additional `find()`/`findall()` methods.
    """

    @abc.abstractmethod
    def list(self):
        pass

    def find(self, **kwargs):
        """
        Find a single item with attributes matching ``**kwargs``.

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if num_matches == 0:
            msg = "No %s matching %s." % (self.resource_class.__name__, kwargs)
            raise exceptions.NotFound(404, msg)
        elif num_matches > 1:
            raise exceptions.NoUniqueMatch
        else:
            return matches[0]

    def findall(self, **kwargs):
        """
        Find all items with attributes matching ``**kwargs``.

        To find volume with size less than equal to 500 GB and id contains
        'ironic'

        kwargs = {'freeSize_le': 500, 'id_has': 'ironic', "UsedSize": 300}

        Operator:
        no operator required for "equal to" checks
        _le: less than equal to
        _ge: greater than equal to
        _has: contains string

        This isn't very efficient: it loads the entire list then filters on
        the Python side.
        """
        found = []
        searches = kwargs.items()

        listing = self.list()

        for obj in listing:
            try:
                for attr, value in searches:
                    if attr.endswith('_eq'):
                        if getattr(obj, attr) == value:
                            found.append(obj)
                    elif attr.endswith('_le'):
                        if getattr(obj, attr) <= value:
                            found.append(obj)
                    elif attr.endswith('_ge'):
                        if getattr(obj, attr) >= value:
                            found.append(obj)
                    elif attr.endswith('_has'):
                        if value in getattr(obj, attr):
                            found.append(obj)
                    else:
                        if getattr(obj, attr) == value:
                            found.append(obj)

            except AttributeError:
                continue

        return found


class Resource(object):

    """
    A resource represents a particular instance of an object (server, flavor,
    etc). This is pretty much just a bag for attributes.

    :param manager: Manager object
    :param info: dictionary representing resource attributes
    :param loaded: prevent lazy-loading if set to True
    """
    HUMAN_ID = False
    NAME_ATTR = 'id'

    def __init__(self, manager, info, loaded=False):
        self.manager = manager
        self._info = info
        self._add_details(info)
        self._loaded = loaded

    @property
    def human_id(self):
        """Subclasses may override this provide a pretty ID which can be used
        for bash completion.
        """
        if self.NAME_ATTR in self.__dict__ and self.HUMAN_ID:
            return strutils.to_slug(getattr(self, self.NAME_ATTR))
        return None

    def _add_details(self, info):
        for (k, v) in six.iteritems(info):
            try:
                setattr(self, k, v)
                self._info[k] = v
            except AttributeError:
                # In this case we already defined the attribute on the class
                pass

    def __getattr__(self, k):
        if k not in self.__dict__:
            # NOTE(rk): disallow lazy-loading if already loaded once
            if not self.is_loaded():
                self.get()
                return self.__getattr__(k)

            raise AttributeError(k)
        else:
            return self.__dict__[k]

    def __repr__(self):
        reprkeys = sorted(k for k in self.__dict__.keys() if k[0] != '_' and
                          k != 'manager')
        info = ", ".join("%s=%s" % (k, getattr(self, k)) for k in reprkeys)
        return "<%s %s>" % (self.__class__.__name__, info)

    def get(self):
        # set_loaded() first ... so if we have to bail, we know we tried.
        self.set_loaded(True)
        if not hasattr(self.manager, 'get'):
            return

        new = self.manager.get(self.id)
        if new:
            self._add_details(new._info)

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if hasattr(self, 'id') and hasattr(other, 'id'):
            return self.id == other.id
        return self._info == other._info

    def is_loaded(self):
        return self._loaded

    def set_loaded(self, val):
        self._loaded = val

    def refresh(self, sleep=None):
        if sleep:
            time.sleep(sleep)
        return self.manager.get(self.id)



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient.tests import utils
from seamicroclient.tests.v2 import fakes
from seamicroclient.v2 import disks


cs = fakes.FakeClient()


class DisksTest(utils.TestCase):

    def test_list_disks(self):
        sl = cs.disks.list()
        cs.assert_called('GET', '/storage/disks')
        [self.assertTrue(isinstance(s, disks.Disk)) for s in sl]

    def test_get_disk(self):
        s = cs.disks.get(1)
        cs.assert_called('GET', '/storage/disks/1')
        self.assertTrue(isinstance(s, disks.Disk))

    def test_disk_power_on(self):
        cs.disks.power_on(1)
        cs.assert_called('PUT', '/storage/disks/1')

    def test_disk_power_off(self):
        cs.disks.power_off(1)
        cs.assert_called('PUT', '/storage/disks/1')

    def test_disk_activate_led(self):
        cs.disks.activate_led(1)
        cs.assert_called('PUT', '/storage/disks/1')

    def test_disk_deactivate_lef(self):
        cs.disks.deactivate_led(1)
        cs.assert_called('PUT', '/storage/disks/1')



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Seamicro Client interface. Handles the REST calls and responses.
"""

import logging
import time

import requests

try:
    import json
except ImportError:
    import simplejson as json

from seamicroclient import exceptions
from seamicroclient import utils


class HTTPClient(object):

    USER_AGENT = 'python-seamicroclient'

    def __init__(self, user, password, auth_url=None,
                 timeout=None, http_log_debug=False, retries=3):
        self.user = user
        self.password = password
        self.api_endpoint = auth_url
        self.auth_url = auth_url.rstrip('/')
        self.version = 'v2.0'
        self.http_log_debug = http_log_debug
        if timeout is not None:
            self.timeout = float(timeout)
        else:
            self.timeout = None
        self.retries = int(retries)

        self.times = []  # [("item", starttime, endtime), ...]

        self._logger = logging.getLogger(__name__)
        if self.http_log_debug and not self._logger.handlers:
            # Logging level is already set on the root logger
            ch = logging.StreamHandler()
            self._logger.addHandler(ch)
            self._logger.propagate = False
            if hasattr(requests, 'logging'):
                rql = requests.logging.getLogger(requests.__name__)
                rql.addHandler(ch)
                # Since we have already setup the root logger on debug, we
                # have to set it up here on WARNING (its original level)
                # otherwise we will get all the requests logging messanges
                rql.setLevel(logging.WARNING)

    def get_timings(self):
        return self.times

    def reset_timings(self):
        self.times = []

    def http_log_req(self, method, url, kwargs):
        if not self.http_log_debug:
            return

        string_parts = ['curl -i']

        string_parts.append(" '%s'" % url)
        string_parts.append(' -X %s' % method)

        for element in kwargs['headers']:
            header = ' -H "%s: %s"' % (element, kwargs['headers'][element])
            string_parts.append(header)

        if 'data' in kwargs:
            string_parts.append(" -d '%s'" % (kwargs['data']))
        self._logger.debug("\nREQ: %s\n" % "".join(string_parts))

    def http_log_resp(self, resp):
        if not self.http_log_debug:
            return
        self._logger.debug(
            "RESP: [%s] %s\nRESP BODY: %s\n",
            resp.status_code,
            resp.headers,
            resp.text)

    def request(self, url, method, **kwargs):
        kwargs.setdefault('headers', kwargs.get('headers', {}))
        kwargs['headers']['User-Agent'] = self.USER_AGENT
        kwargs['headers']['Accept'] = 'application/json'
        if 'body' in kwargs:
            kwargs['headers']['Content-Type'] = 'application/json'
            kwargs['data'] = json.dumps(kwargs['body'])
            del kwargs['body']
        if self.timeout is not None:
            kwargs.setdefault('timeout', self.timeout)

        self.http_log_req(method, url, kwargs)
        resp = requests.request(
            method,
            url,
            **kwargs)
        self.http_log_resp(resp)

        if resp.text:
            # httplib2 returns a connection refused event as a 400 response.
            # To determine if it is a bad request or refused connection we need
            # to check the body.  httplib2 tests check for 'Connection refused'
            # or 'actively refused' in the body, so that's what we'll do.
            if resp.status_code == 400:
                if ('Connection refused' in resp.text or
                        'actively refused' in resp.text):
                    raise exceptions.ConnectionRefused(resp.text)
            try:
                body = json.loads(resp.text)
            except ValueError:
                body = resp.text
        else:
            body = None

        if resp.status_code >= 400:
            raise exceptions.from_response(resp, body, url, method)

        return resp, body

    def _time_request(self, url, method, **kwargs):
        start_time = time.time()
        resp, body = self.request(url, method, **kwargs)
        self.times.append(("%s %s" % (method, url),
                           start_time, time.time()))
        return resp, body

    def _cs_request(self, url, method, **kwargs):
        attempts = 0
        retry_delay = 5
        while True:
            attempts += 1
            if method in ['GET', 'DELETE']:
                url = "%s?username=%s&password=%s" % (url, self.user,
                                                      self.password)
            else:
                kwargs.setdefault('body', {}).update({'username': self.user,
                                                    'password': self.password})
            try:
                resp, body = self._time_request(self.api_endpoint + url,
                                                method,**kwargs)
                return resp, body
            except requests.exceptions.ConnectionError as e:
                if attempts > self.retries:
                    raise
                # Catch a connection refused from requests.request
                # retry again with some time delay
                self._logger.debug("Connection refused: %s" % e)
                self._logger.debug("Failed attempt(%s of %s), "
                                   "retrying in %s seconds" %
                                   (attempts, self.retries,
                                    retry_delay))
                time.sleep(retry_delay)

    def get(self, url, **kwargs):
        return self._cs_request(url, 'GET', **kwargs)

    def post(self, url, **kwargs):
        return self._cs_request(url, 'POST', **kwargs)

    def put(self, url, **kwargs):
        return self._cs_request(url, 'PUT', **kwargs)

    def delete(self, url, **kwargs):
        return self._cs_request(url, 'DELETE', **kwargs)


def get_client_class(version):
    version_map = {
        '2': 'seamicroclient.v2.client.Client',
    }
    try:
        client_path = version_map[str(version)]
    except (KeyError, ValueError):
        msg = "Invalid client version '%s'. must be one of: %s" % (
              (version, ', '.join(version_map.keys())))
        raise exceptions.UnsupportedVersion(msg)

    return utils.import_class(client_path)


def Client(version, *args, **kwargs):
    client_class = get_client_class(version)
    return client_class(*args, **kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Exception definitions.
"""


class UnsupportedVersion(Exception):

    """Indicates that the user is trying to use an unsupported
    version of the API.
    """
    pass


class CommandError(Exception):
    pass


class AuthorizationFailure(Exception):
    pass


class NoUniqueMatch(Exception):
    pass


class AuthSystemNotFound(Exception):

    """When the user specify a AuthSystem but not installed."""

    def __init__(self, auth_system):
        self.auth_system = auth_system

    def __str__(self):
        return "AuthSystemNotFound: %s" % repr(self.auth_system)


class NoTokenLookupException(Exception):

    """This form of authentication does not support looking up
       endpoints from an existing token.
    """
    pass


class EndpointNotFound(Exception):

    """Could not find Service or Region in Service Catalog."""
    pass


class AmbiguousEndpoints(Exception):

    """Found more than one matching endpoint in Service Catalog."""

    def __init__(self, endpoints=None):
        self.endpoints = endpoints

    def __str__(self):
        return "AmbiguousEndpoints: %s" % repr(self.endpoints)


class ConnectionRefused(Exception):

    """
    Connection refused: the server refused the connection.
    """

    def __init__(self, response=None):
        self.response = response

    def __str__(self):
        return "ConnectionRefused: %s" % repr(self.response)


class ClientException(Exception):

    """
    The base exception class for all exceptions this library raises.
    """

    def __init__(self, code, message=None, details=None, request_id=None,
                 url=None, method=None):
        self.code = code
        self.message = message or self.__class__.message
        self.details = details
        self.request_id = request_id
        self.url = url
        self.method = method

    def __str__(self):
        formatted_string = "%s (HTTP %s)" % (self.message, self.code)
        if self.request_id:
            formatted_string += " (Request-ID: %s)" % self.request_id

        return formatted_string


class BadRequest(ClientException):

    """
    HTTP 400 - Bad request: you sent some malformed data.
    """
    http_status = 400
    message = "Bad request"


class Unauthorized(ClientException):

    """
    HTTP 401 - Unauthorized: bad credentials.
    """
    http_status = 401
    message = "Unauthorized"


class Forbidden(ClientException):

    """
    HTTP 403 - Forbidden: your credentials don't give you access to this
    resource.
    """
    http_status = 403
    message = "Forbidden"


class NotFound(ClientException):

    """
    HTTP 404 - Not found
    """
    http_status = 404
    message = "Not found"


class MethodNotAllowed(ClientException):

    """
    HTTP 405 - Method Not Allowed
    """
    http_status = 405
    message = "Method Not Allowed"


class Conflict(ClientException):

    """
    HTTP 409 - Conflict
    """
    http_status = 409
    message = "Conflict"


class OverLimit(ClientException):

    """
    HTTP 413 - Over limit: you're over the API limits for this time period.
    """
    http_status = 413
    message = "Over limit"

    def __init__(self, *args, **kwargs):
        try:
            self.retry_after = int(kwargs.pop('retry_after'))
        except (KeyError, ValueError):
            self.retry_after = 0

        super(OverLimit, self).__init__(*args, **kwargs)


class RateLimit(OverLimit):

    """
    HTTP 429 - Rate limit: you've sent too many requests for this time period.
    """
    http_status = 429
    message = "Rate limit"


# NotImplemented is a python keyword.
class HTTPNotImplemented(ClientException):

    """
    HTTP 501 - Not Implemented: the server does not support this operation.
    """
    http_status = 501
    message = "Not Implemented"


# In Python 2.4 Exception is old-style and thus doesn't have a __subclasses__()
# so we can do this:
#     _code_map = dict((c.http_status, c)
#                      for c in ClientException.__subclasses__())
#
# Instead, we have to hardcode it:
_error_classes = [BadRequest, Unauthorized, Forbidden, NotFound,
                  MethodNotAllowed, Conflict, OverLimit, RateLimit,
                  HTTPNotImplemented]
_code_map = dict((c.http_status, c) for c in _error_classes)


def from_response(response, body, url, method=None):
    """
    Return an instance of an ClientException or subclass
    based on an requests response.

    Usage::

        resp, body = requests.request(...)
        if resp.status_code != 200:
            raise exception_from_response(resp, rest.text)
    """
    kwargs = {
        'code': response.status_code,
        'method': method,
        'url': url,
    }

    if body:
        message = "n/a"
        details = "n/a"

        if hasattr(body, 'keys'):
            error = body[list(body)[1]]
            message = error.get('message', None)
            details = error.get('details', None)

        kwargs['message'] = message
        kwargs['details'] = details

    cls = _code_map.get(response.status_code, ClientException)
    return cls(**kwargs)



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient.tests import utils
from seamicroclient.tests.v2 import fakes
from seamicroclient.v2 import pools


cs = fakes.FakeClient()


class PoolsTest(utils.TestCase):

    def test_list_pools(self):
        pl = cs.pools.list()
        cs.assert_called('GET', '/storage/pools')
        [self.assertTrue(isinstance(s, pools.Pool)) for s in pl]

    def test_get_pool(self):
        p = cs.pools.get(1)
        cs.assert_called('GET', '/storage/pools/1')
        self.assertTrue(isinstance(p, pools.Pool))

    def test_create_pool(self):
        cs.pools.create(1, "pool-name", [1, 5, 6])
        cs.assert_called('PUT', '/storage/pools/1/pool-name')

    def test_delete_pool(self):
        cs.pools.delete('1/pool-name')
        cs.assert_called('DELETE', '/storage/pools/1/pool-name')

    def test_mount_pool(self):
        cs.pools.mount(1)
        cs.assert_called('PUT', '/storage/pools/1')

    def test_unmount_pool(self):
        cs.pools.unmount(1)
        cs.assert_called('PUT', '/storage/pools/1')



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import pkg_resources
import sys
import uuid

import six

from seamicroclient import exceptions
from seamicroclient.openstack.common import strutils


def env(*args, **kwargs):
    """
    returns the first environment variable set
    if none are non-empty, defaults to '' or keyword arg default
    """
    for arg in args:
        value = os.environ.get(arg, None)
        if value:
            return value
    return kwargs.get('default', '')


def find_resource(manager, name_or_id, **find_args):
    """Helper for the _find_* methods."""
    # first try to get entity as integer id
    try:
        return manager.get(int(name_or_id))
    except (TypeError, ValueError, exceptions.NotFound):
        pass

    # now try to get entity as uuid
    try:
        tmp_id = strutils.safe_encode(name_or_id)
        if six.PY3:
            tmp_id = tmp_id.decode()
        uuid.UUID(tmp_id)
        return manager.get(tmp_id)
    except (TypeError, ValueError, exceptions.NotFound):
        pass

    # for str id which is not uuid (for Flavor search currently)
    if getattr(manager, 'is_alphanum_id_allowed', False):
        try:
            return manager.get(name_or_id)
        except exceptions.NotFound:
            pass

    try:
        try:
            return manager.find(human_id=name_or_id, **find_args)
        except exceptions.NotFound:
            pass

        # finally try to find entity by name
        try:
            resource = getattr(manager, 'resource_class', None)
            name_attr = resource.NAME_ATTR if resource else 'name'
            kwargs = {name_attr: name_or_id}
            kwargs.update(find_args)
            return manager.find(**kwargs)
        except exceptions.NotFound:
            msg = "No %s with a name or ID of '%s' exists." % \
                (manager.resource_class.__name__.lower(), name_or_id)
            raise exceptions.CommandError(msg)
    except exceptions.NoUniqueMatch:
        msg = ("Multiple %s matches found for '%s', use an ID to be more"
               " specific." % (manager.resource_class.__name__.lower(),
                               name_or_id))
        raise exceptions.CommandError(msg)


def _format_field_name(attr):
    """Format an object attribute in a human-friendly way."""
    # Split at ':' and leave the extension name as-is.
    parts = attr.rsplit(':', 1)
    name = parts[-1].replace('_', ' ')
    # Don't title() on mixed case
    if name.isupper() or name.islower():
        name = name.title()
    parts[-1] = name
    return ': '.join(parts)


def _make_field_formatter(attr, filters=None):
    """
    Given an object attribute, return a formatted field name and a
    formatter suitable for passing to print_list.

    Optionally pass a dict mapping attribute names to a function. The function
    will be passed the value of the attribute and should return the string to
    display.
    """
    filter_ = None
    if filters:
        filter_ = filters.get(attr)

    def get_field(obj):
        field = getattr(obj, attr, '')
        if field and filter_:
            field = filter_(field)
        return field

    name = _format_field_name(attr)
    formatter = get_field
    return name, formatter


class HookableMixin(object):

    """Mixin so classes can register and run hooks."""
    _hooks_map = {}

    @classmethod
    def add_hook(cls, hook_type, hook_func):
        if hook_type not in cls._hooks_map:
            cls._hooks_map[hook_type] = []

        cls._hooks_map[hook_type].append(hook_func)

    @classmethod
    def run_hooks(cls, hook_type, *args, **kwargs):
        hook_funcs = cls._hooks_map.get(hook_type) or []
        for hook_func in hook_funcs:
            hook_func(*args, **kwargs)


def safe_issubclass(*args):
    """Like issubclass, but will just return False if not a class."""

    try:
        if issubclass(*args):
            return True
    except TypeError:
        pass

    return False


def import_class(import_str):
    """Returns a class from a string including module and class."""
    mod_str, _sep, class_str = import_str.rpartition('.')
    __import__(mod_str)
    return getattr(sys.modules[mod_str], class_str)


def _load_entry_point(ep_name, name=None):
    """Try to load the entry point ep_name that matches name."""
    for ep in pkg_resources.iter_entry_points(ep_name, name=name):
        try:
            return ep.load()
        except (ImportError, pkg_resources.UnknownExtra, AttributeError):
            continue


def is_integer_like(val):
    """Returns validation of a value as an integer."""
    try:
        value = int(val)
        return True
    except (TypeError, ValueError, AttributeError):
        return False



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient.tests import utils
from seamicroclient.tests.v2 import fakes
from seamicroclient.v2 import volumes


cs = fakes.FakeClient()


class VolumesTest(utils.TestCase):

    def test_list_volumes(self):
        vl = cs.volumes.list()
        cs.assert_called('GET', '/storage/volumes')
        [self.assertTrue(isinstance(s, volumes.Volume)) for s in vl]

    def test_get_volume(self):
        p = cs.volumes.get(1)
        cs.assert_called('GET', '/storage/volumes/1')
        self.assertTrue(isinstance(p, volumes.Volume))

    def test_create_volume(self):
        v = cs.volumes.create(2, '0/p0_0', '1')
        cs.assert_called('PUT', '/storage/volumes/0/p0_0/1')
        self.assertTrue(isinstance(v, volumes.Volume))



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient.tests import utils
from seamicroclient.tests.v2 import fakes
from seamicroclient.v2 import servers


cs = fakes.FakeClient()


class ServersTest(utils.TestCase):

    def test_list_servers(self):
        sl = cs.servers.list()
        cs.assert_called('GET', '/servers')
        [self.assertTrue(isinstance(s, servers.Server)) for s in sl]

    def test_get_server(self):
        s = cs.servers.get(1)
        cs.assert_called('GET', '/servers/1')
        self.assertTrue(isinstance(s, servers.Server))

    def test_server_power_on(self):
        cs.servers.power_on(1)
        cs.assert_called('PUT', '/servers/1')

    def test_server_power_off(self):
        cs.servers.power_off(1)
        cs.assert_called('PUT', '/servers/1?action=power-off')

    def test_server_reset(self):
        cs.servers.reset(1)
        cs.assert_called('PUT', '/servers/1')

    def test_server_set_tagged_vlan(self):
        cs.servers.set_tagged_vlan(1, '12-12')
        cs.assert_called('PUT', '/servers/1/nic/0/taggedVlans')

    def test_server_unset_tagged_vlan(self):
        cs.servers.unset_tagged_vlan(1, '12-12')
        cs.assert_called('PUT', '/servers/1/nic/0/taggedVlans')

    def test_server_set_untagged_vlan(self):
        cs.servers.set_untagged_vlan(1, '12-12')
        cs.assert_called('PUT', '/servers/1/nic/0/untaggedVlans')

    def test_server_unset_untagged_vlan(self):
        cs.servers.unset_untagged_vlan(1, '12-12')
        cs.assert_called('PUT', '/servers/1/nic/0/untaggedVlans')

    def test_server_attach_volume_default(self):
        cs.servers.attach_volume(1, '1/p6-6/vol1')
        cs.assert_called('PUT', '/servers/1/vdisk/0')

    def test_server_attach_volume_with_vdisk(self):
        cs.servers.attach_volume(1, '1/p6-6/vol1', vdisk=3)
        cs.assert_called('PUT', '/servers/1/vdisk/3')

    def test_server_detach_volume(self):
        cs.servers.detach_volume(1, vdisk=0)
        cs.assert_called('DELETE', '/servers/1/vdisk/0')

    def test_set_boot_order(self):
        cs.servers.set_boot_order(1, 'pxe')
        cs.assert_called('PUT', '/servers/1')



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import uuid

from seamicroclient.tests import utils
from seamicroclient.v2 import Client


cs = Client("admin", "seamicro", "http://chassis/v2.0")


class PoolsTest(utils.TestCase):

    def test_list_pool(self):
        pool_list = cs.pools.list()
        self.assertTrue(len(pool_list) > 0)

    def test_list_pool_with_filter(self):
        filters = {'id': 'p6-'}
        for pool in cs.pools.list(filters):
            for k, v in filters.iteritems():
                self.assertIn(v, getattr(pool, k))

    def test_list_pool_with_filter_no_match(self):
        filters = {'id': str(uuid.uuid4())}
        for pool in cs.pools.list(filters):
            for k, v in filters.iteritems():
                self.assertNotIn(getattr(pool, k), v)

    def test_get_pool(self):
        pool_id = cs.pools.list()[0].id
        pool = cs.pools.get(pool_id)
        self.assertEqual(pool.id, pool_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
System interface.
"""

from seamicroclient import base


class System(base.Resource):
    HUMAN_ID = True

    def switchover(self, mxcard=None, **kwargs):
        return self.manager.switchover(self, mxcard, **kwargs)

    def writemem(self, **kwargs):
        return self.manager.writemem(self, **kwargs)

    def add_segment(self, vlan_id, **kwargs):
        return self.manager.add_segment(self, vlan_id, **kwargs)

    def remove_segment(self, vlan_id, **kwargs):
        return self.manager.remove_segment(self, vlan_id, **kwargs)

class SystemManager(base.ManagerWithFind):
    resource_class = System

    def list(self, filters=None):
        """
        Get a list of system properties.

        :rtype: list of :class:`System`
        """
        return self._list("/chassis/system", filters=filters)

    def switchover(self, system, mxcard=None, **kwargs):
        """
        Switchover system to different mxcard
        """
        url = "/chassis/system/switchover"
        body = {}
        if mxcard is not None:
            body = {'newActive': mxcard}
        return self.api.client.put(url, body=body)

    def writemem(self, system, **kwargs):
        """
        Write current system config to flash memory
        This will persist even after reboot of chassis
        """
        url = "/chassis/system/writeMem"
        return self.api.client.put(url, body={})

    def reload(self, system, **kwargs):
        """
        Reload the chassis and start the boot image
        """
        url = "/chassis/system/reload"
        return self.api.client.put(url, body={})

    def add_segment(self, system, vlan_id, **kwargs):
        """
        Create Global Network
        """
        url = "/chassis/system/vlans"
        if vlan_id is not None:
            action_params = {}
            action_params.update({'add': vlan_id})
            self.run_hooks('modify_body_for_action', action_params, **kwargs)
            return self.api.client.put(url, body=action_params)

    def remove_segment(self, system, vlan_id, **kwargs):
        """
        Remove Global Network
        """
        url = "/chassis/system/vlans"
        if vlan_id is not None:
            action_params = {}
            action_params.update({'remove': vlan_id})
            self.run_hooks('modify_body_for_action', action_params, **kwargs)
            return self.api.client.put(url, body=action_params)



import json

from seamicroclient.v2 import client

seamicro = client.Client("admin", "seamicro", "http://172.16.0.25/v2.0")

servers = seamicro.servers.list()

for server in servers:
    print json.dumps(server)
    print "\n\n"



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
SMCard interface.
"""

from seamicroclient import base


class SMCard(base.Resource):
    HUMAN_ID = True


class SMCardManager(base.ManagerWithFind):
    resource_class = SMCard

    def get(self, smcard):
        """
        Get a smcard.

        :param smcard: ID of the :class:`SMCard` to get.
        :rtype: :class:`SMCard`
        """
        return self._get(base.getid(smcard),
                         "/chassis/smcard/%s" % base.getid(smcard))

    def list(self, filters=None):
        """
        Get a list of smcard properties.

        :rtype: list of :class:`SMCard`
        """
        return self._list("/chassis/smcard", filters=filters)



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from seamicroclient.tests import utils
from seamicroclient.v2 import Client


cs = Client("admin", "seamicro", "http://chassis/v2.0")


class VolumesTest(utils.TestCase):

    @staticmethod
    def create_volume(volume_size=2, pool=None):
        return cs.volumes.create(volume_size, pool)

    def test_list_volume(self):
        volume_list = cs.volumes.list()
        self.assertTrue(len(volume_list) > 0)

    def test_get_volume(self):
        volume_id = cs.volumes.list()[0].id
        volume = cs.volumes.get(volume_id)
        self.assertEqual(volume.id, volume_id)

    def test_create_volume(self):
        pool = cs.pools.list()[0]
        volume_size = 2
        volume_id = self.create_volume(volume_size, pool)
        self.assertIn(pool.id, volume_id)
        cs.volumes.delete(volume_id)

    def test_delete_volume(self):
        pool = cs.pools.list()[0]
        volume = self.create_volume(pool=pool)
        cs.volumes.delete(volume)
        volume = cs.volumes.get(volume)
        self.assertEqual(volume.actualSize, 0)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

import mock
import requests


from seamicroclient import client
from seamicroclient import exceptions
from seamicroclient.tests import utils


fake_response = utils.TestResponse({
    "status_code": 200,
    "text": '{"hi": "there"}',
})
mock_request = mock.Mock(return_value=(fake_response))

refused_response = utils.TestResponse({
    "status_code": 400,
    "text": '[Errno 111] Connection refused',
})
refused_mock_request = mock.Mock(return_value=(refused_response))

bad_req_response = utils.TestResponse({
    "status_code": 400,
    "text": '',
})
bad_req_mock_request = mock.Mock(return_value=(bad_req_response))


def get_client():
    cl = client.HTTPClient("username", "password", "http://example.com")
    return cl


def get_authed_client():
    cl = get_client()
    cl.auth_url = "http://example.com"
    cl.auth_token = "token"
    cl.user = "user"
    cl.password = "password"
    return cl


class ClientTest(utils.TestCase):

    def test_get(self):
        cl = get_authed_client()

        @mock.patch.object(requests.Session, "request", mock_request)
        @mock.patch('time.time', mock.Mock(return_value=1234))
        def test_get_call():
            resp, body = cl.get("/hi")
            headers = {'Accept': 'application/json',
                       'User-Agent': 'python-seamicroclient'}
            mock_request.assert_called_with(
                "GET",
                "http://example.com/hi?username=%s&password=%s" % (cl.user,
                                                                cl.password),
                headers=headers)
            # Automatic JSON parsing
            self.assertEqual(body, {"hi": "there"})

        test_get_call()

    def test_post(self):
        cl = get_authed_client()

        @mock.patch.object(requests.Session, "request", mock_request)
        def test_post_call():
            body = {'k1': 'v1', 'k2': 'v2', 'authtoken': cl.auth_token}
            cl.post("/hi", body=body)
            headers = {'Content-Type': 'application/json', 'Accept':
                       'application/json',
                       'User-Agent': 'python-seamicroclient'}
            mock_request.assert_called_with(
                "POST",
                "http://example.com/hi",
                headers=headers,
                data=json.dumps(body))

        test_post_call()

    def test_connection_refused(self):
        cl = get_client()

        @mock.patch.object(requests.Session, "request", refused_mock_request)
        def test_refused_call():
            self.assertRaises(exceptions.ConnectionRefused, cl.get, "/hi")

        test_refused_call()

    def test_bad_request(self):
        cl = get_client()

        @mock.patch.object(requests.Session, "request", bad_req_mock_request)
        def test_refused_call():
            self.assertRaises(exceptions.BadRequest, cl.get, "/hi")

        test_refused_call()

    def test_client_logger(self):
        cl1 = client.HTTPClient("username", "password",
                                "http://example.com",
                                http_log_debug=True)
        self.assertEqual(len(cl1._logger.handlers), 1)

        cl2 = client.HTTPClient("username", "password",
                                "http://example.com",
                                http_log_debug=True)
        self.assertEqual(len(cl2._logger.handlers), 1)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Interfaces object.
"""

from seamicroclient import base


class Interface(base.Resource):
    HUMAN_ID = True

    def shutdown(self, **kwargs):
        return self.manager.shutdown(self, **kwargs)

    def no_shutdown(self, **kwargs):
        return self.manager.no_shutdown(self, **kwargs)

    def add_tagged_vlan(self, vlan_id, **kwargs):
        return self.manager.add_tagged_vlan(self, vlan_id, **kwargs)

    def remove_tagged_vlan(self, vlan_id, **kwargs):
        return self.manager.remove_tagged_vlan(self, vlan_id, **kwargs)

    def add_untagged_vlan(self, vlan_id, **kwargs):
        return self.manager.add_untagged_vlan(self, vlan_id, **kwargs)

    def remove_untagged_vlan(self, vlan_id, **kwargs):
        return self.manager.remove_untagged_vlan(self, vlan_id, **kwargs)


class InterfaceManager(base.ManagerWithFind):
    resource_class = Interface

    def get(self, interface):
        """
        Get a interface.

        :param interface: ID of the :class:`Interface` to get.
        :rtype: :class:`Interface`
        """
        return self._get(base.getid(interface),
                         "/interfaces/%s" % base.getid(interface))

    def list(self, filters=None):
        """
        Get a list of interfaces.

        :rtype: list of :class:`Interface`
        """
        return self._list("/interfaces", filters=filters)

    def shutdown(self, interface, **kwargs):
        """
        Shutdown the specified network Interface
        """
        url = "/interfaces/%s/shutdown" % base.getid(interface)
        body = {'value': True}
        return self.api.client.put(url, body=body)

    def no_shutdown(self, interface, **kwargs):
        """
        Start the specified network Interface
        """
        url = "/interfaces/%s/shutdown" % base.getid(interface)
        body = {'value': False}
        return self.api.client.put(url, body=body)

    def add_tagged_vlan(self, interface, vlan_id, **kwargs):
        """
        Add tagged vlan for the given Interface
        """
        url = '/interfaces/%s/vlans/taggedVlans' % base.getid(interface)
        if isinstance(vlan_id, list):
            vlan_id = map(lambda x: str(x), vlan_id)
            body = {'add': ','.join(vlan_id)}
        else:
            body = {'add': str(vlan_id)}
        return self.api.client.put(url, body=body)

    def remove_tagged_vlan(self, interface, vlan_id, **kwargs):
        """
        Remove tagged vlan for the given Interface
        """
        url = '/interfaces/%s/vlans/taggedVlans' % base.getid(interface)
        body = {'remove': str(vlan_id)}
        return self.api.client.put(url, body=body)

    def add_untagged_vlan(self, interface, vlan_id, **kwargs):
        """
        Add untagged vlan for the given Interface
        """
        url = '/interfaces/%s/vlans/untaggedVlans' % base.getid(interface)
        body = {'add': str(vlan_id)}
        return self.api.client.put(url, body=body)

    def remove_untagged_vlan(self, interface, vlan_id, **kwargs):
        """
        Remove untagged vlan for the given Interface
        """
        url = '/interfaces/%s/vlans/untaggedVlans' % base.getid(interface)
        body = {'remove': str(vlan_id)}
        return self.api.client.put(url, body=body)



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient.tests import utils
from seamicroclient.tests.v2 import fakes
from seamicroclient.v2 import interfaces


cs = fakes.FakeClient()


class InterfacesTest(utils.TestCase):

    def test_list_interfaces(self):
        sl = cs.interfaces.list()
        cs.assert_called('GET', '/interfaces')
        [self.assertTrue(isinstance(s, interfaces.Interface)) for s in sl]

    def test_interface_get(self):
        cs.interfaces.get(1)
        cs.assert_called('GET', '/interfaces/1')

    def test_interface_shutdown(self):
        cs.interfaces.shutdown(1)
        cs.assert_called('PUT', '/interfaces/1/shutdown')

    def test_interface_no_shutdown(self):
        cs.interfaces.no_shutdown(1)
        cs.assert_called('PUT', '/interfaces/1/shutdown')

    def test_interface_add_taggedvlan_list(self):
        cs.interfaces.add_tagged_vlan(1, [1, 2, 3])
        cs.assert_called('PUT', '/interfaces/1/vlans/taggedVlans')

    def test_interface_add_taggedvlan_single(self):
        cs.interfaces.add_tagged_vlan(1, '23-25')
        cs.assert_called('PUT', '/interfaces/1/vlans/taggedVlans')

    def test_interface_remove_taggedvlan(self):
        cs.interfaces.remove_tagged_vlan(1, '23-25')
        cs.assert_called('PUT', '/interfaces/1/vlans/taggedVlans')

    def test_interface_add_untaggedvlan(self):
        cs.interfaces.add_untagged_vlan(1, '23')
        cs.assert_called('PUT', '/interfaces/1/vlans/untaggedVlans')

    def test_interface_remove_untaggedvlan(self):
        cs.interfaces.remove_untagged_vlan(1, '23')
        cs.assert_called('PUT', '/interfaces/1/vlans/untaggedVlans')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Disks interface.
"""

from seamicroclient import base


class Disk(base.Resource):
    HUMAN_ID = True

    def power_on(self, **kwargs):
        return self.manager.power_on(self, **kwargs)

    def power_off(self, **kwargs):
        return self.manager.power_off(self, **kwargs)

    def active_led(self, **kwargs):
        return self.manager.active_led(self, **kwargs)

    def deactivate_led(self, **kwargs):
        return self.manager.deactivate_led(self, **kwargs)


class DiskManager(base.ManagerWithFind):
    resource_class = Disk

    def get(self, disk):
        """
        Get a disk.

        :param disk: ID of the :class:`Disk` to get.
        :rtype: :class:`Disk`
        """
        return self._get(base.getid(disk),
                         "/storage/disks/%s" % base.getid(disk))

    def list(self, filters=None):
        """
        Get a list of disks.

        :rtype: list of :class:`Disk`
        """
        return self._list("/storage/disks", filters=filters)

    def power_off(self, disk, **kwargs):
        """
        Power off the specified Disk
        """
        url = "/storage/disks/%s" % base.getid(disk)
        body = {'action': 'power-off'}
        return self.api.client.put(url, body=body)

    def power_on(self, disk, **kwargs):
        """
        Power on the specified Disk
        """
        url = "/storage/disks/%s" % base.getid(disk)
        body = {'action': 'power-on'}
        return self.api.client.put(url, body=body)

    def activate_led(self, disk, **kwargs):
        """
        Activate LED of the specified Disk
        """
        url = "/storage/disks/%s" % base.getid(disk)
        body = {'action': 'activate-led'}
        return self.api.client.put(url, body=body)

    def deactivate_led(self, disk, **kwargs):
        """
        De-activate LED of the specified Disk
        """
        url = "/storage/disks/%s" % base.getid(disk)
        body = {'action': 'deactivate-led'}
        return self.api.client.put(url, body=body)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import fixtures
import requests
import testtools


class TestCase(testtools.TestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        if (os.environ.get('OS_STDOUT_CAPTURE') == 'True' or
                os.environ.get('OS_STDOUT_CAPTURE') == '1'):
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if (os.environ.get('OS_STDERR_CAPTURE') == 'True' or
                os.environ.get('OS_STDERR_CAPTURE') == '1'):
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))


class TestResponse(requests.Response):

    """
    Class used to wrap requests.Response and provide some
    convenience to initialize with a dict
    """

    def __init__(self, data):
        self._text = None
        super(TestResponse, self)
        if isinstance(data, dict):
            self.status_code = data.get('status_code', None)
            self.headers = data.get('headers', None)
            # Fake the text attribute to streamline Response creation
            self._text = data.get('text', None)
        else:
            self.status_code = data

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

    @property
    def text(self):
        return self._text



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Volume interface.
"""

import binascii
import os

from seamicroclient import base


class Volume(base.Resource):
    HUMAN_ID = True

    def delete(self):
        self.manager.delete(self)


class VolumeManager(base.ManagerWithFind):
    resource_class = Volume

    def get(self, volume):
        """
        Get a volume.

        :param volume: ID of the :class:`Volume` to get.
        :rtype: :class:`Volume`
        """
        return self._get(base.getid(volume),
                         "/storage/volumes/%s" % base.getid(volume))

    def list(self):
        """
        Get a list of volumes.

        :rtype: list of :class:`Volume`
        """
        return self._list("/storage/volumes")

    def create(self, size, pool, volume_id=None, **kwargs):
        """
        Create a volume of the given size in the given pool.

        :param volume_id: ID of the :class: `Volume` to create
        :param pool: Object  of the :class: `Pool` in which the volume will be
                     created.
        :param size: Size of the new volume in GB.
        """
        create_params = {}
        if volume_id is None:
            volume_id = str(binascii.b2a_hex(os.urandom(6)))
        if pool and volume_id and size:
            create_params = {'volume-size': str(size)}
            resource_url = "%s/%s" % (base.getid(pool), volume_id)
            return self._create(resource_url, create_params)

    def delete(self, volume):
        self._delete("/storage/volume/%s" % base.getid(volume))

    def _create(self, resource_url, body, **kwargs):
        """
        Create a volume
        """
        body.update({"action": "create"})
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/storage/volumes/%s' % resource_url
        return self._update(url, body=body)

    def _action(self, action, volume, info=None, **kwargs):
        """
        Perform a volume "action" -- .
        """
        body = {"action": action}
        body.update(info)
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/storage/volumes/%s' % base.getid(volume)
        return self.api.client.put(url, body=body)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import seamicroclient.client
from seamicroclient.tests import utils
import seamicroclient.v2.client


class ClientTest(utils.TestCase):

    def test_client_with_timeout(self):
        instance = seamicroclient.client.HTTPClient(user='user',
                                                    password='password',
                                                    timeout=2,
                                                    auth_url="http://test")
        self.assertEqual(instance.timeout, 2)

    def test_get_client_class_v2(self):
        output = seamicroclient.client.get_client_class('2')
        self.assertEqual(output, seamicroclient.v2.client.Client)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Chassis interface.
"""

from seamicroclient import base


class Chassis(base.Resource):
    HUMAN_ID = True

    def writemem(self, **kwargs):
        return self.manager.writemem(self, **kwargs)


class ChassisManager(base.ManagerWithFind):
    resource_class = Chassis

    def list(self, filters=None):
        """
        Get a list of chassis properties.

        :rtype: list of :class:`Chassis`
        """
        return self._list("/chassis", filters=filters)

    def writemem(self, chassis, **kwargs):
        """
        Write current chassis config to flash memory
        This will persist even after reboot of chassis
        """
        url = "/chassis/system/writeMem"
        return self.api.client.put(url, body={})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Server interface.
"""

from seamicroclient import base


TAGGED_VLAN = "taggedVlans"
UNTAGGED_VLAN = "untaggedVlan"


class Server(base.Resource):
    HUMAN_ID = True

    def power_on(self, using_pxe=False):
        self.manager.power_on(self, using_pxe)

    def power_off(self, force=False):
        self.manager.power_off(self, force)

    def reset(self, using_pxe=False):
        self.manager.reset(self, using_pxe)

    def set_tagged_vlan(self, vlan_id, nics=[], **kwargs):
        self.manager.set_tagged_vlan(self, vlan_id, nics, **kwargs)

    def unset_tagged_vlan(self, vlan_id, nics=[], **kwargs):
        self.manager.unset_tagged_vlan(self, vlan_id, nics, **kwargs)

    def set_untagged_vlan(self, vlan_id, nics=[], **kwargs):
        self.manager.set_untagged_vlan(self, vlan_id, nics, **kwargs)

    def unset_untagged_vlan(self, vlan_id, nics=[], **kwargs):
        self.manager.unset_untagged_vlan(self, vlan_id, nics, **kwargs)

    def attach_volume(self, volume, vdisk=0, **kwargs):
        self.manager.attach_volume(self, volume, vdisk, **kwargs)

    def detach_volume(self, vdisk=0, **kwargs):
        self.manager.detach_volume(self, vdisk, **kwargs)

    def set_boot_order(self, boot_order="hd0", **kwargs):
        self.manager.set_boot_order(self, boot_order, **kwargs)

    def get_boot_order(self, **kwargs):
        return self.manager.get_boot_order(self, **kwargs)

class ServerManager(base.ManagerWithFind):
    resource_class = Server

    def get(self, server):
        """
        Get a server.

        :param server: ID of the :class:`Server` to get.
        :rtype: :class:`Server`
        """
        return self._get(base.getid(server),
                         "/servers/%s" % base.getid(server))

    def list(self):
        """
        Get a list of servers.

        :rtype: list of :class:`Server`
        """
        return self._list("/servers")

    def attach_volume(self, server, volume, vdisk=0, **kwargs):
        """
        Attach volume to vdisk # to given server

        :param server: The :class:`Server` (or its ID) to power on.
        :param volume: The :class:`Volume` (or its ID) that is to be attached.
        :param vdisk: The vdisk number of the server to attach volume to.
        :
        """
        body = {"value": volume}
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/servers/%s/vdisk/%s' % (base.getid(server), vdisk)
        return self.api.client.put(url, body=body)

    def detach_volume(self, server, vdisk=0, **kwargs):
        """
        Detach volume attached to vdisk # of given server

        :param server: The :class:`Server` (or its ID) to power on.
        :param vdisk: The vdisk number of the server to detach volume to.
        :
        """
        url = '/servers/%s/vdisk/%s' % (base.getid(server), vdisk)
        return self._delete(url)

    def power_on(self, server, using_pxe=False, **kwargs):
        """
        Power on a server.

        :param server: The :class:`Server` (or its ID) to power on.
        :param using_pxe: power on server and use pxe boot.
        """
        action_params = {}
        if using_pxe:
            action_params = {"using-pxe": using_pxe}
        self._action('power-on', server, action_params)

    def power_off(self, server, force=False, **kwargs):
        """
        Power off a server.

        :param server: The :class:`Server` (or its ID) to power off.
        :param force: force the server to power off.
        """
        action = 'power-off'
        url = '/servers/%s?action=%s' % (base.getid(server), action)
        if force:
            url = '%s&force=true' % url
        return self.api.client.put(url, body={})

    def reset(self, server, using_pxe=False, **kwargs):
        """
        Reset power of a server.

        :param server: The :class:`Server` (or its ID) to power on.
        :param using_pxe: reset and power on server and use pxe boot.
        """
        action_params = {}
        if using_pxe:
            action_params = {"using-pxe": using_pxe}
        self._action('reset', server, action_params)

    def set_tagged_vlan(self, server, vlan_id, nics=[], **kwargs):
        """
        Set the tagged vlan id for the server.

        :param server: The :class:`Server` (or its ID) to power on.
        :param vlan_id: The tagged vlan id for the server.
        """
        self._handle_vlan(server, vlan_id, TAGGED_VLAN, nics, **kwargs)

    def unset_tagged_vlan(self, server, vlan_id, nics=[], **kwargs):
        """
        Unset the tagged vlan id for the server.

        :param server: The :class:`Server` (or its ID) to power on.
        :param vlan_id: The tagged vlan id for the server.
        """
        self._handle_vlan(server, vlan_id, TAGGED_VLAN, nics, unset=True, **kwargs)

    def set_untagged_vlan(self, server, vlan_id, nics=[], **kwargs):
        """
        Set the untagged vlan id for the server.

        :param server: The :class:`Server` (or its ID) to power on.
        :param vlan_id: The untagged vlan id for the server.
        """
        self._handle_vlan(server, vlan_id, UNTAGGED_VLAN, nics, **kwargs)

    def unset_untagged_vlan(self, server, vlan_id, nics=[], **kwargs):
        """
        Unset the untagged vlan id for the server.

        :param server: The :class:`Server` (or its ID) to power on.
        :param vlan_id: The untagged vlan id for the server.
        """
        self._handle_vlan(server, vlan_id, UNTAGGED_VLAN, nics,
                          unset=True, **kwargs)

    def _handle_vlan(self, server, vlan_id, vlan_type, nics=[], unset=False, **kwargs):
        """
        Set/Unset tagged/untagged vlan id for the server.

        :param server: The :class:`Server` (or its ID) to power on.
        :param vlan_id: The tagged vlan id for the server.
        :param vlan_type: tagged-vlan or untagged-vlan type.
        :param unset: Boolean flag to unset the vlan_id for the server.
        """
        if vlan_id is not None:

            action_params = {}
            if unset:
                action_params.update({'remove': vlan_id})
            else:
                action_params.update({'add': vlan_id})
            self.run_hooks('modify_body_for_action', action_params, **kwargs)

            if nics:
                for nic in nics:
                    url = '/servers/%s/nic/%s/%s' % (base.getid(server), nic[3:], vlan_type)
                    self.api.client.put(url, body=action_params)
                return
            else:
                u = '/servers/%s/nic' % (base.getid(server))
                nics = self._list(u)
                for nic in nics:
                    url = '/servers/%s/nic/%s/%s' % (base.getid(server), nic.id, vlan_type)
                    self.api.client.put(url, body=action_params)
                return

    def set_boot_order(self, server, boot_order="hd0", **kwargs):
        """
        Set bios boot order for the server

        :param server: The :class:`Server` (or its ID)
        :param boot_order: The boot order for the server
        """
        action_params = {'boot-order': boot_order}
        if boot_order == "pxe":
            action_params.update({'boot-order': 'pxe,hd0'})

        return self._action('set-bios-boot-order', server, action_params)

    def get_boot_order(self, server):
        url = '/server/%s/bios/bootOrder' % base.getid(server)
        ret,val = self.api.client.get(url)
        return (ret , val.split(" ")[0])

    def _action(self, action, server, info=None, **kwargs):
        """
        Perform a server "action" -- power-on/power-off/reset/etc.
        """
        body = {"action": action}
        body.update(info)
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = '/servers/%s' % base.getid(server)
        return self.api.client.put(url, body=body)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from seamicroclient import base
from seamicroclient.tests import utils
from seamicroclient.tests.v2 import fakes
from seamicroclient.v2 import servers


cs = fakes.FakeClient()


class BaseTest(utils.TestCase):

    def test_resource_repr(self):
        r = base.Resource(None, dict(foo="bar", baz="spam"))
        self.assertEqual(repr(r), "<Resource baz=spam, foo=bar>")

    def test_getid(self):
        self.assertEqual(base.getid(4), 4)

        class TmpObject(object):
            id = 4
        self.assertEqual(base.getid(TmpObject), 4)

    def test_resource_lazy_getattr(self):
        f = servers.Server(cs.servers, {'id': 1})
        self.assertEqual(f.name, 'sample-server')
        cs.assert_called('GET', '/servers/1')

        # Missing stuff still fails after a second get
        self.assertRaises(AttributeError, getattr, f, 'blahblah')

    def test_eq(self):
        # Two resources of the same type with the same id: equal
        r1 = base.Resource(None, {'id': 1, 'name': 'hi'})
        r2 = base.Resource(None, {'id': 1, 'name': 'hello'})
        self.assertEqual(r1, r2)

        # Two resoruces of different types: never equal
        r1 = base.Resource(None, {'id': 1})
        r2 = servers.Server(None, {'id': 1})
        self.assertNotEqual(r1, r2)

        # Two resources with no ID: equal if their info is equal
        r1 = base.Resource(None, {'name': 'joe', 'age': 12})
        r2 = base.Resource(None, {'name': 'joe', 'age': 12})
        self.assertEqual(r1, r2)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Scard interface.
"""

from seamicroclient import base


class Scard(base.Resource):
    HUMAN_ID = True

    def set_management_mode(self, mode, force=False, **kwargs):
        return self.manager.set_management_mode(self, mode, force, **kwargs)

    def volume_mode(self, value, **kwargs):
        if value:
            return self.manager.set_management_mode(self, 'volume', **kwargs)
        else:
            return self.manager.set_management_mode(self, 'disk', **kwargs)


class ScardManager(base.ManagerWithFind):
    resource_class = Scard

    def list(self, filters=None):
        """
        Get a list of scard properties.

        :rtype: list of :class:`Scard`
        """
        return self._list("/chassis/scards", filters=filters)

    def get(self, scard, **kwargs):
        """
        Get a specific scard.

        :rtype: Instance of :class:`Scard`
        """
        return self._get(base.getid(scard),
                         '/chassis/scard/%s' % base.getid(scard))

    def set_management_mode(self, scard, mode, force=False, **kwargs):
        """
        Set management mode of the specified scard
        """
        url = "/chassis/scard/%s/mgmtMode" % base.getid(scard)
        body = {'value': mode}
        return self.api.client.put(url, body=body)
